<?php
namespace tmc\revisionmanager\src\AdminPages\PageSettings;

/**
 * @author jakubkuranda@gmail.com
 * Date: 06.03.2018
 * Time: 11:58
 */

use shellpress\v1_3_84\src\Shared\AdminPageFramework\AdminPage;
use tmc\revisionmanager\src\App;

class PageSettings extends AdminPage {

    /**
     * Declaration of current element.
     */
    public function setUp() {

        //  ----------------------------------------
        //  Definition
        //  ----------------------------------------

        $this->pageFactory->addSubMenuItem(
            array(
                'title'         =>  'Revision Manager TMC',
                'page_slug'     =>  $this->pageSlug
            )
        );

        $this->pageFactory->setInPageTabTag( 'h2' );

        //  ----------------------------------------
        //  Tabs
        //  ----------------------------------------

        new TabGeneral(     $this->pageFactory, $this->pageSlug, 'general' );
        new TabTools(       $this->pageFactory, $this->pageSlug, 'tools' );
        new TabLicense(     $this->pageFactory, $this->pageSlug, 'license' );

        //  ----------------------------------------
        //  Filters
        //  ----------------------------------------

        add_filter( 'script_' . $this->pageSlug,                    array( $this, '_f_pageScriptPreventEnter' ) );
        add_filter( 'footer_left_' . $this->pageFactoryClassName,   array( $this, '_f_footerLeftContent' ) );
        add_filter( 'footer_right_' . $this->pageFactoryClassName,  array( $this, '_f_footerRightContent' ) );

    }

    /**
     * Called while current component is loaded.
     */
    public function load() {

        //  ----------------------------------------
        //  Styles
        //  ----------------------------------------

        $this->pageFactory->enqueueStyle(
            App::s()->getUrl( 'assets/css/style.css' ),
            $this->pageSlug,
            '',
            array(
               'version'        =>  App::s()->getFullPluginVersion()
            )
        );

        $this->pageFactory->enqueueStyle(
            App::s()->getUrl( 'vendor/tmc/shellpress/assets/css/AdminPageFramework/SPAdminPageFramework.css' ),
            $this->pageSlug,
            '',
            array(
                'version'        =>  App::s()->getFullPluginVersion()
            )
        );

        //  ----------------------------------------
        //  Scripts
        //  ----------------------------------------

        $this->pageFactory->enqueueScript(
            App::s()->getUrl( 'assets/js/script.js' ),
            $this->pageSlug,
            '',
            array(
                'version'        =>  App::s()->getFullPluginVersion()
            )
        );

    }

    //  ================================================================================
    //  FILTERS
    //  ================================================================================

    /**
     * Javascript code which will prevent form submission on 'enter' key press.
     *
     * @param string $script
     *
     * @return string
     */
    public function _f_pageScriptPreventEnter( $script ) {

        $script .= <<<JS

            jQuery( document ).ready( function( $ ){
                
                $( '#tmcapf-form' ).on( 'keyup keypress', function( e ) {
                    
                    var keyCode = e.keyCode || e.which;
                    
                    if( keyCode === 13 ) { 
                        e.preventDefault();
                        return false;
                    }
                    
                } );
                
            } );

JS;

        return $script;

    }

	/**
	 * @param string $html
	 *
	 * @return string
	 */
    public function _f_footerLeftContent( $html ) {

    	$html = sprintf(
    		        __( 'If you like Revision Manager TMC please leave us a %1$s rating. A huge thanks in advance! | Your feedback is welcome |', 'rm_tmc' ),
		            sprintf( '<a target="_blank" href="https://wordpress.org/support/plugin/revision-manager-tmc/reviews/?filter=5#postform">★★★★★</a>' )
	            );

    	return $html;

    }

	/**
	 * @param string $html
	 *
	 * @return string
	 */
	public function _f_footerRightContent( $html ) {

		return '';

	}

}