<?php

namespace tmc\revisionmanager\src\AdminPages\PageSettings;

/**
 * @author jakubkuranda@gmail.com
 * Date: 08.03.2018
 * Time: 09:39
 */

use shellpress\v1_3_84\src\Shared\AdminPageFramework\AdminPageTab;
use shellpress\v1_3_84\src\Shared\Front\Models\HtmlElement;
use tmc\revisionmanager\src\App;
use tmc\revisionmanager\src\Components\Notifications;
use TMC_v1_0_6_RadioRevealFieldType;
use WP_Post_Type;
use WP_Role;

class TabGeneral extends AdminPageTab {

    /**
     * Declaration of current element.
     */
    public function setUp() {

        //  ----------------------------------------
        //  Definition
        //  ----------------------------------------

        $this->pageFactory->addInPageTab(
            array(
                'page_slug'     =>  $this->pageSlug,
                'tab_slug'      =>  $this->tabSlug,
                'title'         =>  __( 'General settings', 'rm_tmc' ),
            )
        );

    }

    /**
     * Called while current component is loaded.
     */
    public function load() {

    	//  ----------------------------------------
    	//  Custom field types
    	//  ----------------------------------------

	    App::s()->requireFile( 'vendor/tmc/admin-page-framework/custom-field-types/radio-reveal-field-type/RadioRevealFieldType.php', 'TMC_v1_0_6_RadioRevealFieldType' );

	    new TMC_v1_0_6_RadioRevealFieldType();

        //  ----------------------------------------
        //  Modifications
        //  ----------------------------------------

        add_filter( 'fields_' . $this->pageFactoryClassName,                array( $this, '_f_redefineField_postTypes_chosen' ) );
        add_filter( 'fields_' . $this->pageFactoryClassName,                array( $this, '_f_redefineField_capabilities_excludedEmails' ) );
        add_filter( 'fields_' . $this->pageFactoryClassName,                array( $this, '_f_redefineField_notifications' ) );

        //  ----------------------------------------
        //  Sections
        //  ----------------------------------------

        $this->pageFactory->addSettingSections(
            array(
                'section_id'            =>  'capabilities',
                'page_slug'             =>  $this->pageSlug,
                'tab_slug'              =>  $this->tabSlug,
                'title'                 =>  __( 'Capabilities', 'rm_tmc' )
            ),
	        array(
		        'section_id'            =>  'merging',
		        'page_slug'             =>  $this->pageSlug,
		        'tab_slug'              =>  $this->tabSlug,
		        'title'                 =>  __( 'Merging', 'rm_tmc' )
	        ),
	        array(
		        'section_id'            =>  'wpDifferences',
		        'page_slug'             =>  $this->pageSlug,
		        'tab_slug'              =>  $this->tabSlug,
		        'title'                 =>  __( 'Differences for WordPress', 'rm_tmc' )
	        ),
	        array(
		        'section_id'            =>  'acfDifferences',
		        'page_slug'             =>  $this->pageSlug,
		        'tab_slug'              =>  $this->tabSlug,
		        'title'                 =>  __( 'Differences for Advanced Custom Fields', 'rm_tmc' ),
		        'description'           =>  App::i()->acfDifferences->isCurrentAcfVersionCompatible()
			                                ? null
			                                : sprintf( '<span style="color: #bd2130;">%1$s</span>', __( 'Revision Manager requires ACF >= v5', 'rm_tmc' ) )
	        ),
            array(
                'section_id'            =>  'postTypes',
                'page_slug'             =>  $this->pageSlug,
                'tab_slug'              =>  $this->tabSlug,
                'title'                 =>  __( 'Post types', 'rm_tmc' )
            ),
            array(
                'section_id'            =>  'notifications',
                'page_slug'             =>  $this->pageSlug,
                'tab_slug'              =>  $this->tabSlug,
                'title'                 =>  __( 'Notifications', 'rm_tmc' )
            ),
            array(
                'section_id'            =>  'submit',
                'page_slug'             =>  $this->pageSlug,
                'tab_slug'              =>  $this->tabSlug,
                'title'                 =>  null,
                'save'                  =>  false
            )
        );

        //  ----------------------------------------
        //  Fields
        //  ----------------------------------------

        $this->pageFactory->addSettingFields(
            'capabilities', //  section_id
            array(
                'field_id'              =>  'capCopy',
                'type'                  =>  'select',
                'title'                 =>  __( 'Copy creation capability', 'rm_tmc' ),
                'label'                 =>  $this->getAllCapabilitiesNames()
            ),
            array(
                'field_id'              =>  'capAccept',
                'type'                  =>  'select',
                'title'                 =>  __( 'Acceptation capability', 'rm_tmc' ),
                'label'                 =>  $this->getAllCapabilitiesNames()
            ),
            array(
                'field_id'              =>  'roleNotification',
                'type'                  =>  'select',
                'title'                 =>  __( 'Role for notification', 'rm_tmc' ),
                'label'                 =>  $this->getAllRolesNames(),
                'attributes'            =>  array()
            ),
            array(
                'field_id'              =>  'excludedEmails',
                'type'                  =>  'textarea',
                'title'                 =>  __( 'Excluded e-mails', 'rm_tmc' ),
                'attributes'            =>  array(),
                'tip'                   =>  __( 'These addresses will never receive any notification.', 'rm_tmc' ),
                'description'           =>  __( 'Comma separated e-mail addresses. Example: mail@host1.com, mail@host2.com', 'rm_tmc' )
            )
        );

	    $this->pageFactory->addSettingFields(
		    'merging', //  section_id
		    array(
			    'field_id'              =>  'mergeDate',
			    'type'                  =>  'radio',
			    'title'                 =>  __( 'Merge date with revision', 'rm_tmc' ),
			    'label'                 =>  array(
			    	'0'                     =>  __( 'No, keep original post\'s date', 'rm_tmc' ),
			    	'1'                     =>  __( 'Yes, after revision acceptance, replace date', 'rm_tmc' )
			    )
		    )
	    );

        $this->pageFactory->addSettingFields(
        	'wpDifferences',  //  section_id
	        array(
	        	'field_id'              =>  'displayPostTitle',
		        'type'                  =>  'radio',
		        'title'                 =>  __( 'Post title', 'rm_tmc' ),
		        'label'                 =>  array(
		        	'1'                     =>  __( 'Display changes in post title', 'rm_tmc' ),
			        '0'                     =>  __( 'Do not show', 'rm_tmc' )
		        )
	        ),
	        array(
		        'field_id'              =>  'displayPostContent',
		        'type'                  =>  'radio',
		        'title'                 =>  __( 'Post content', 'rm_tmc' ),
		        'label'                 =>  array(
			        '1'                     =>  __( 'Display changes in post content', 'rm_tmc' ),
			        '0'                     =>  __( 'Do not show', 'rm_tmc' )
		        )
	        )
        );

	    $this->pageFactory->addSettingFields(
		    'acfDifferences', //  section_id
		    array(
			    'field_id'              =>  'markChanges',
			    'type'                  =>  'radioreveal',
			    'title'                 =>  __( 'Differences', 'rm_tmc' ),
			    'label'                 =>  array(
			    	'1'                     =>  __( 'Mark changed fields', 'rm_tmc' ),
				    '0'                     =>  __( 'Do not show', 'rm_tmc' )
			    ),
			    'reveals'               =>  array(
			    	'1'                 =>  '#fieldrow-acfDifferences_changeMarkColor, #fieldrow-acfDifferences_newMarkColor'
			    ),
			    'description'           =>  __( 'If there is a difference between original post field and clone, there will be a mark on the side of field.', 'rm_tmc' )
		    ),
		    array(
		    	'field_id'              =>  'changeMarkColor',
			    'type'                  =>  'color',
			    'title'                 =>  __( 'Color of change mark', 'rm_tmc' ),
		    ),
		    array(
			    'field_id'              =>  'newMarkColor',
			    'type'                  =>  'color',
			    'title'                 =>  __( 'Color of new mark', 'rm_tmc' ),
			    'tip'                   =>  __( 'Accent for field without previous value.', 'rm_tmc' )
		    )
	    );

        $this->pageFactory->addSettingFields(
            'postTypes',    //  section_id
            array(
                'field_id'              =>  'chosen',
                'type'                  =>  'checkbox',
                'title'                 =>  null,
                'label'                 =>  $this->getAllPostTypesNames(),
                'before_field'          =>  sprintf( '<p>%1$s</p><br/>', __( 'Plugin will only work on chosen post types.', 'rm_tmc' ) )
            )
        );

        $this->pageFactory->addSettingFields(
            'notifications',    //  section_id
            array(
				'field_id'              =>  'type',
	            'type'                  =>  'radioreveal',
	            'title'                 =>  __( 'Type of notifications', 'rm_tmc' ),
	            'label'                 =>  array(
	            	'everySingle'           =>  __( 'Every single revision', 'rm_tmc' ),
		            'collective'            =>  __( 'All revisions in one e-mail (one per day)', 'rm_tmc' )
	            ),
	            'before_field'          =>  sprintf( '<p>%1$s</p><br/>', __( 'I want to receive notification about:', 'rm_tmc' ) ),
	            'reveals'               =>  array(
	            	'everySingle'            =>  '#fieldrow-notifications_thisAccLimit'
	            )
            ),
	        array(
		        'field_id'              =>  'whoReceives',
		        'type'                  =>  'radio',
		        'title'                 =>  __( 'Who receives notifications?', 'rm_tmc' ),
		        'label'                 =>  array(
			        'all'                   =>  __( 'All authorized users', 'rm_tmc' ),
			        'authors'               =>  __( 'Only original authors', 'rm_tmc' )
		        )
	        ),
            array(
                'field_id'              =>  'title',
                'type'                  =>  'text',
                'title'                 =>  __( 'Title', 'rm_tmc' ),
                'attributes'            =>  array(
                    'class'                 =>  'regular-text'
                )
            ),
            array(
                'field_id'              =>  'content',
                'type'                  =>  'textarea',
                'title'                 =>  __( 'Content', 'rm_tmc' ),
                'rich'                  =>  array(
                    'tinymce'               =>  array(
                        'wp_autoresize_on'      =>  true,
                        'toolbar1'              =>  'formatselect,bold,italic,underline,forecolor,bullist,numlist,link,unlink',
                        'width'                 =>  500
                    )
                ),
                'before_field'          =>  $this->getDisplayOfTemplatePicker(),
                'after_field'           =>  $this->getDisplayOfTemplateCodes(),
                'attributes'            =>  array()
            ),
            array(
                'field_id'              =>  'emailAddress',
                'type'                  =>  'email',
                'title'                 =>  __( 'Quick test', 'rm_tmc' ),
                'attributes'            =>  array(
                    'class'                 =>  'regular-text'
                ),
                'value'                 =>  wp_get_current_user()->user_email,
                'description'           =>  __( 'Insert your e-mail address and send preview of notification.', 'rm_tmc' ),
                'after_input'           =>  sprintf( '
                                                <button
                                                class="button"
                                                disabled="disabled"
                                                data-quick-test-action="%1$s"
                                                data-target-input-id="#notifications_emailAddress__0"
                                                data-title-input-id="#notifications_title__0"
                                                data-content-tinymce-id="#wp-notifications_content__0-wrap"
                                                data-content-textarea-id="#notifications_content__0"
                                                >%2$s</button>',
                                                Notifications::SEND_QUICK_EMAIL_TEST_ACTION_NAME,
                                                __( 'Send test', 'rm_tmc' )
                                            ),
                'save'                  =>  false
            )
        );

        $this->pageFactory->addSettingFields(
            'submit',    //  section_id
            array(
                'field_id'              =>  'submitForm',
                'type'                  =>  'submit',
                'title'                 =>  null,
                'value'                 =>  __( 'Update settings', 'rm_tmc' )
            )
        );

    }

    //  ================================================================================
    //  FILTERS
    //  ================================================================================

    /**
     * @param array $fields
     *
     * @return array
     */
    public function _f_redefineField_capabilities_excludedEmails( $fields ){

        if( ! App::i()->license->isLicenseActive() ){

            $content = sprintf( '<a class="pro-link" href="%1$s" target="_blank">%2$s</a>',
                App::TMC_SHOP_URL,
                __( 'PRO version gives you ability to exclude certain e-mail addresses from notifications.', 'rm_tmc' )
            );

            $fields['capabilities']['excludedEmails']['description']    = null;
            $fields['capabilities']['excludedEmails']['content']        = $content;

        }

        return $fields;

    }

    /**
     * @param array $fields
     *
     * @return array
     */
    public function _f_redefineField_postTypes_chosen( $fields ) {

        if( ! App::i()->license->isLicenseActive() ){

            //  Attributes

            $attrsDefinition    = array();
            $postTypeNames      = $this->getAllPostTypesNames();

            foreach( $postTypeNames as $key => $value ){

                //  Disable non-default post type

                if( $key !== 'post' ){
                    $attrsDefinition[ $key ] = array(
                        'disabled'  =>  'disabled'
                    );
                }

            }

            $fields['postTypes']['chosen']['attributes'] = $attrsDefinition;

            //  Info

            $fields['postTypes']['chosen']['after_field'] = sprintf( '<div style="clear: both; padding-top: 1em;"><a class="pro-link" href="%1$s" target="_blank">%2$s</a></div>',
                                                                App::TMC_SHOP_URL,
                                                                __( 'You can choose every custom post type in PRO version.', 'rm_tmc' )
                                                            );

        }

        return $fields;

    }

    /**
     * @param array $fields
     *
     * @return array
     */
    public function _f_redefineField_notifications( $fields ) {

        if( ! App::i()->license->isLicenseActive() ){

        	//  ----------------------------------------
        	//  Type
        	//  ----------------------------------------

	        $fields['notifications']['type']['attributes']['field']['class']    = 'form-disabled';
	        $fields['notifications']['type']['after_field']                     = sprintf( '<div style="clear: both; padding-top: 1em;"><a class="pro-link" href="%1$s" target="_blank">%2$s</a></div>',
		        App::TMC_SHOP_URL,
		        __( 'You can choose alternative notifications type in PRO version.', 'rm_tmc' )
	        );

	        //  ----------------------------------------
	        //  Who receives
	        //  ----------------------------------------

	        $fields['notifications']['whoReceives']['attributes']['field']['class']    = 'form-disabled';
	        $fields['notifications']['whoReceives']['after_field']                     = sprintf( '<div style="clear: both; padding-top: 1em;"><a class="pro-link" href="%1$s" target="_blank">%2$s</a></div>',
		        App::TMC_SHOP_URL,
		        __( 'You can limit notifications to original authors in PRO version.', 'rm_tmc' )
	        );

        	//  ----------------------------------------
        	//  Content
        	//  ----------------------------------------

            $previewHtml = sprintf( '<div><div style="display: inline-block;">%1$s</div><p style="padding-top: 1em;"><a class="pro-link" href="%2$s" target="_blank">%3$s</a></p></div>',
                App::i()->settings->getNotificationContent(),
                App::TMC_SHOP_URL,
                __( 'PRO version gives you ability to edit this content.', 'rm_tmc' )
            );

            $fields['notifications']['content']['after_field']                  = $previewHtml;
            $fields['notifications']['content']['attributes']['field']['style'] = 'display: none;';

        }

        return $fields;

    }

    //  ================================================================================
    //  FUNCTIONALITY
    //  ================================================================================

    /**
     * @return string[]
     */
    protected function getAllCapabilitiesNames() {

        $role               = get_role( 'administrator' );    //  Administrator is used for listing all possible capabilities
        $capabilitiesNames  = array();

        if( $role ){
            foreach( $role->capabilities as $key => $value ) {
                $capabilitiesNames[$key] = $key;
            }
        }

        return $capabilitiesNames;

    }

    /**
     * @return string[]
     */
    protected function getAllRolesNames() {

        $roles = wp_roles()->role_objects;
        $names = wp_roles()->role_names;

        $rolesNames         = array();
        $rolesNames['']     = __( '- Do not send -', 'rm_tmc' );

        foreach( $roles as $role ) {    /** @var WP_Role $role */
            $rolesNames[ $role->name ] = array_key_exists( $role->name, $names ) ? $names[ $role->name ] : $role->name;
        }

        return $rolesNames;

    }

    /**
     * @return string[]
     */
    protected function getAllPostTypesNames() {

        /** @var WP_Post_Type[] $postTypesObjects */
        $postTypesObjects = get_post_types( array( 'show_ui' => true ), 'objects' );

        $postTypes      = array();
        $excludeList    = array( 'attachment' );

        foreach( $postTypesObjects as $postType ){
            if( ! in_array( $postType->name, $excludeList ) ){  //  Exclude if in list

                $postTypes[ $postType->name ] = sprintf( '<span  style="display: inline-block; width: 150px; margin-right: 1em;">%1$s</span> <i>( %2$s )</i>',
                    $postType->label,
                    $postType->name
                );

            }
        }

        return $postTypes;

    }

    /**
     * @return string
     */
    protected function getDisplayOfTemplatePicker() {

        //  ----------------------------------------
        //  Prepare data
        //  ----------------------------------------

        $templates = array(
            'default_mail',
            'blue_mail',
            'mono_mail',
            'light_mail',
            'aqua_mail',
            'moonlight_mail'
        );

        $templatesDirUrl = App::s()->getUrl( '/assets/emailTemplates' );

        //  ----------------------------------------
        //  Build display
        //  ----------------------------------------

        $html = '<div class="template_picker">';

        foreach( $templates as $templateName ){

            $html .= '<div class="cell">';

            $templateHtmlObj = new HtmlElement( 'a' );
            $templateHtmlObj->addAttributes( array(
                'href'              =>  App::i()->license->isLicenseActive() ? '#' : App::TMC_SHOP_URL,
                'data-template-url' =>  $templatesDirUrl . '/' . $templateName . '.html?ver=' . App::s()->getFullPluginVersion(),
                'class'             =>  App::i()->license->isLicenseActive() ? 'template' : 'template disabled',
                'target'            =>  '_blank'
            ) );

            $imgUrl = $templatesDirUrl . '/' . $templateName . '.jpg';

            $html .= $templateHtmlObj->setContent( sprintf( '<img src="%1$s" alt="">', $imgUrl ) )->getDisplay();
            $html .= '</div>';

        }

        $html .= '</div>';

        return $html;

    }

    /**
     * @return string
     */
    public function getDisplayOfTemplateCodes() {

        $html = <<<HTML
        
        <div style="clear: both; padding-top: 1em;">
            <p><b>Use codes below to personalize your e-mail notification:</b></p>

            <ul>
                <li><code>%post_url%</code> - URL of post visualization.</li>
                <li><code>%post_title%</code> - Title of this post.</li>
                <li><code>%post_admin_url%</code> - URL of admin page, where you can accept changes.</li>
                <li><code>%author_name%</code> - Author of this revision.</li>
                <li><code>%author_mail%</code> - His/her e-mail address.</li>
                <li><code>%site_url%</code> - Dynamic link of your website.</li>
            </ul>
        </div>

HTML;

        return $html;

    }

}