<?php
namespace tmc\revisionmanager\src;

/**
 * @author jakubkuranda@gmail.com
 * Date: 06.03.2018
 * Time: 10:21
 */

use shellpress\v1_3_84\ShellPress;
use tmc\revisionmanager\src\Components\DashboardWidget;
use tmc\revisionmanager\src\Components\AcfDifferences;
use tmc\revisionmanager\src\Components\License;
use tmc\revisionmanager\src\Components\Notifications;
use tmc\revisionmanager\src\Components\Revisions;
use tmc\revisionmanager\src\Components\Settings;
use tmc\revisionmanager\src\Components\System;
use tmc\revisionmanager\src\Components\Upgrade;
use tmc\revisionmanager\src\Components\Utilities;
use TMC_revision_manager_AdminPageFramework;

class App extends ShellPress {

    const TMC_SHOP_URL = 'https://themastercut.co/plugins/revision-manager-tmc/?utm_source=client&utm_medium=plugin&utm_campaign=revision-manager-tmc';

    /** @var License */
    public $license;

    /** @var Settings */
    public $settings;

    /** @var Revisions */
    public $revisions;

    /** @var Utilities */
    public $utilities;

    /** @var Upgrade */
    public $upgrade;

    /** @var DashboardWidget */
    public $dashboardWidget;

    /** @var AcfDifferences */
    public $acfDifferences;

    /** @var Notifications */
    public $notifications;

    /** @var System */
    public $system;

    /**
     * Called automatically after core is ready.
     *
     * @return void
     */
    protected function onSetUp() {

        //  ----------------------------------------
        //  Components
        //  ----------------------------------------

        $this->upgrade          = new Upgrade( $this );
        $this->license          = new License( $this );
        $this->settings         = new Settings( $this );
        $this->utilities        = new Utilities( $this );
        $this->notifications    = new Notifications( $this );
        $this->revisions        = new Revisions( $this );
        $this->dashboardWidget  = new DashboardWidget( $this );
        $this->acfDifferences   = new AcfDifferences( $this );
        $this->system           = new System( $this );

        //  ----------------------------------------
        //  Actions
        //  ----------------------------------------

        add_action( 'admin_enqueue_scripts', array( $this, '_a_enqueueAdminStyles' ) );

        //  ----------------------------------------
        //  Admin pages
        //  ----------------------------------------

        if( is_admin() && ! wp_doing_ajax() && ! wp_doing_cron() ){   //  Keep it lightweight

            App::s()->requireFile( 'vendor/tmc/admin-page-framework/admin-page-framework.php', 'TMC_v1_0_6_AdminPageFramework' );

            new TMC_revision_manager_AdminPageFramework(
                App::s()->options->getOptionsKey(),                     //  Options
                App::s()->getMainPluginFile(),                          //  Main file
                'manage_options',                                       //  Capability
                App::s()->getPrefix()                                   //  Text domain
            );

        }

    }

    //  ================================================================================
    //  ACTIONS
    //  ================================================================================

    /**
     * @return void
     */
    public function _a_enqueueAdminStyles() {

        wp_enqueue_style( 'rm_tmc_shared', App::s()->getUrl( '/assets/css/shared.css' ), array(), App::s()->getFullPluginVersion() );

    }

}